require 'oauth2'

require 'fortytwo/configuration'
require 'fortytwo/endpoints/user'
require 'fortytwo/endpoints/users'
require 'fortytwo/endpoints/user_sessions'

module FortyTwo
  class Client
    API_HOST = 'https://api.intra.42.fr'
    REQUEST_CLASSES = [FortyTwo::Endpoint::User,
                       FortyTwo::Endpoint::Users,
                       FortyTwo::Endpoint::UserSessions]

    attr_reader :configuration

    def initialize(options = nil)
      @configuration = nil
      define_request_methods

      unless options.nil?
        @configuration = Configuration.new(options)
        check_api_keys
      end
    end

    def configure
      @configuration = Configuration.new
      yield(@configuration)
      check_api_keys
    end

    def check_api_keys
      if configuration.nil? || configuration.invalid?
        @configuration = nil
      else
        @configuration.freeze
      end
    end

    def token
      return @token if instance_variable_defined?(:@token)

      check_api_keys
      api_key = @configuration.api_key
      api_secret = @configuration.api_secret
      client = OAuth2::Client.new(api_key, api_secret, site: API_HOST)
      client.client_credentials.get_token
    end

    private

    def define_request_methods
      REQUEST_CLASSES.each do |request_class|
        endpoint_instance = request_class.new(self)
        create_methods_from_instance(endpoint_instance)
      end
    end

    def create_methods_from_instance(instance)
      instance.public_methods(false).each do |method_name|
        add_method(instance, method_name)
      end
    end

    def add_method(instance, method_name)
      define_singleton_method(method_name) do |*args|
        instance.public_send(method_name, *args)
      end
    end
  end
end
