require 'active_support/all'

require 'fortytwo/responses/user_sessions'

module FortyTwo
  module Endpoint
    class UserSessions

      def initialize(client)
        @client = client
      end

      def user_sessions(username, start_time = last_monday, end_time = right_now, params = { per_page: 100 })
        Response::UserSessions.new(user_sessions_request(username, start_time, end_time, params))
      end

      private

      def user_sessions_request(username, start_time, end_time, params)
        @client.token.get("/v2/users/#{username}/locations?range[begin_at]=#{start_time},#{end_time}", params: params).parsed
      end

      def last_monday
        Time.current.beginning_of_week.to_s.split(" ")[0...-1].join("T")
      end

      def right_now
        Time.current.to_s.split(" ")[0...-1].join("T")
      end
    end
  end
end
